package org.zeroturnaround.javarebel.integration.spring;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.config.PropertyResourceConfigurer;
import org.zeroturnaround.javarebel.LoggerFactory;

public class PropertyResourceConfigurerManager {
  private static Set propertyResourceConfigurers = Collections.synchronizedSet(new HashSet());
  
  private static Method postProcessBeanFactoryMethod;
  
  static {
    try {
      postProcessBeanFactoryMethod = PropertyResourceConfigurer.class.getDeclaredMethod(
          "_postProcessBeanFactory", new Class[] {
              ConfigurableListableBeanFactory.class});
    } catch (SecurityException e) {
      LoggerFactory.getInstance().errorEcho(e);
    } catch (NoSuchMethodException e) {
      LoggerFactory.getInstance().errorEcho(e);
    }
  }
  
  public static void register(PropertyResourceConfigurer prc, ConfigurableListableBeanFactory clbc) {
    propertyResourceConfigurers.add(new PropertyPlaceholderConfigurerEntry(clbc, prc));
  }
  
  public static void reconfigure() {
    /*
     * Rerun PropertyResourceConfigurers
     */
    for (Iterator i = new HashSet(propertyResourceConfigurers).iterator(); i.hasNext();) {
      PropertyPlaceholderConfigurerEntry ppce = (PropertyPlaceholderConfigurerEntry) i.next();
      try {
        ppce.reconfigure();
      } 
      catch (Throwable e) {
        if (e instanceof Error)
          throw (Error) e;
        if (e instanceof RuntimeException)
          throw (RuntimeException) e;
        throw new RuntimeException(e);
      }
    }
  }
  
  private static class PropertyPlaceholderConfigurerEntry {
    private PropertyResourceConfigurer prc;
    private ConfigurableListableBeanFactory clbc;
    
    public PropertyResourceConfigurer getConfigurer() {
      return prc;
    }

    public ConfigurableListableBeanFactory getBeanFactory() {
      return clbc;
    }

    public PropertyPlaceholderConfigurerEntry(
        ConfigurableListableBeanFactory clbc, PropertyResourceConfigurer prc) {
      this.clbc = clbc;
      this.prc = prc;
    }
    
    public void reconfigure() throws Throwable {
      try {
        postProcessBeanFactoryMethod.invoke(prc, new Object[] {clbc});
      } catch (IllegalAccessException e) {
        LoggerFactory.getInstance().errorEcho(e);
        throw e;
      } catch (InvocationTargetException e) {
        LoggerFactory.getInstance().errorEcho(e.getTargetException());
        throw e.getTargetException();
      }
    }
  }
}
